package handlers

import (
	"docker-event-handler/events"
	"errors"
	"fmt"
	"gopkg.in/ini.v1"
	"strings"
	"text/template"
)

type Container struct {
	Action   []string
	Name     []string
	Image    []string
	ID       []string
	Hostname []string
	Label    map[string]*string
	Run      *template.Template
}

func (h Container) Matches(event events.Event) bool {
	e, ok := event.(events.Container)
	if !ok {
		return false
	}

	if !stringListMatches(h.Action, e.Action) {
		return false
	}

	if !stringListMatches(h.ID, e.ID) {
		return false
	}

	if e.Container == nil {
		if len(h.Name) > 0 || len(h.Image) > 0 || len(h.Hostname) > 0 || len(h.Label) > 0 {
			return false
		}
	} else {
		if !stringListMatches(h.Name, e.Container.Name) {
			return false
		}
		if !stringListMatches(h.Image, e.Container.Image) {
			return false
		}
		if !stringListMatches(h.Hostname, e.Container.Config.Hostname) {
			return false
		}
		if !labelsMatch(h.Label, e.Container.Config.Labels) {
			return false
		}
	}

	return true
}

func (h Container) Invoke(event events.Event) error {
	e := event.(events.Container) // enforce that the event is a container events
	return runTemplatedScript(h.Run, e)
}

func readContainerFromConfig(section *ini.Section) (Container, error) {
	var handler Container
	var err error
	handler.Label = make(map[string]*string)
	for _, key := range section.Keys() {
		val := key.String()
		switch key.Name() {
		case "Action":
			handler.Action = append(handler.Action, val)
			break
		case "Name":
			handler.Name = append(handler.Name, val)
			break
		case "Image":
			handler.Image = append(handler.Image, val)
			break
		case "ID":
			handler.ID = append(handler.ID, val)
			break
		case "Hostname":
			handler.Hostname = append(handler.Hostname, val)
			break
		case "Label":
			parts := strings.SplitN(val, "=", 2)
			var k string
			var v *string
			if len(parts) == 2 {
				k = parts[0]
				v = &parts[1]
			} else {
				k = parts[0]
				v = nil
			}
			handler.Label[k] = v
			break
		case "Run":
			if handler.Run != nil {
				return handler, errors.New(fmt.Sprintf("duplicate key %v in section %v", key.Name(), section.Name()))
			}
			handler.Run, err = template.New("Run").Parse(val)
			if err != nil {
				return handler, err
			}
			break
		default:
			return handler, errors.New(fmt.Sprintf("unknown key %v in section %v", key.Name(), section.Name()))
		}
	}
	return handler, nil
}
